@@ -1,3 +1,24 @@
+/**
+ * Distribution License:
+ * BibleDesktop is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License, version 2 as published by
+ * the Free Software Foundation. This program is distributed in the hope
+ * that it will be useful, but WITHOUT ANY WARRANTY; without even the
+ * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * The License is available on the internet at:
+ *       http://www.gnu.org/copyleft/gpl.html
+ * or by writing to:
+ *      Free Software Foundation, Inc.
+ *      59 Temple Place - Suite 330
+ *      Boston, MA 02111-1307, USA
+ *
+ * Copyright: 2005
+ *     The copyright to this program is held by it's authors.
+ *
+ * ID: $Id: Desktop.java,v 1.52 2006/01/05 23:17:31 dmsmith Exp $
+ */
 package org.crosswire.bibledesktop.desktop;
 
 import java.awt.BorderLayout;
@@ -16,6 +37,8 @@ import java.util.Map;
 import java.util.ResourceBundle;
 import java.util.Set;
 
+import javax.swing.BorderFactory;
+import javax.swing.ButtonGroup;
 import javax.swing.ImageIcon;
 import javax.swing.JCheckBoxMenuItem;
 import javax.swing.JComponent;
@@ -25,17 +48,23 @@ import javax.swing.JMenuBar;
 import javax.swing.JOptionPane;
 import javax.swing.JPanel;
 import javax.swing.JPopupMenu;
+import javax.swing.JRadioButtonMenuItem;
 import javax.swing.JSplitPane;
+import javax.swing.SwingUtilities;
 import javax.swing.WindowConstants;
 
 import org.crosswire.bibledesktop.book.BibleViewPane;
 import org.crosswire.bibledesktop.book.DictionaryPane;
+import org.crosswire.bibledesktop.book.DisplaySelectEvent;
+import org.crosswire.bibledesktop.book.DisplaySelectListener;
+import org.crosswire.bibledesktop.book.DisplaySelectPane;
 import org.crosswire.bibledesktop.display.BookDataDisplay;
 import org.crosswire.bibledesktop.display.URLEvent;
 import org.crosswire.bibledesktop.display.URLEventListener;
 import org.crosswire.bibledesktop.util.ConfigurableSwingConverter;
 import org.crosswire.common.config.ChoiceFactory;
 import org.crosswire.common.config.Config;
+import org.crosswire.common.history.History;
 import org.crosswire.common.progress.Job;
 import org.crosswire.common.progress.JobManager;
 import org.crosswire.common.swing.CatchingThreadGroup;
@@ -69,38 +98,15 @@ import org.crosswire.jsword.util.Project;
 import org.jdom.Document;
 
 /**
- * A container for various tools, particularly the BibleGenerator and
- * the Tester. These tools are generally only of use to developers, and
- * not to end users.
+ * The Desktop is the user's view of BibleDesktop.
  *
- * <p>2 Things to think about, if you change the LaF when you have run
- * some tests already, then the window can grow quite a lot. Also do we
- * want to disable the Exit button if work is going on?</p>
- *
- * <p><table border='1' cellPadding='3' cellSpacing='0'>
- * <tr><td bgColor='white' class='TableRowColor'><font size='-7'>
- *
- * Distribution Licence:<br />
- * JSword is free software; you can redistribute it
- * and/or modify it under the terms of the GNU General Public License,
- * version 2 as published by the Free Software Foundation.<br />
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.<br />
- * The License is available on the internet
- * <a href='http://www.gnu.org/copyleft/gpl.html'>here</a>, or by writing to:
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
- * MA 02111-1307, USA<br />
- * The copyright to this program is held by it's authors.
- * </font></td></tr></table>
- * @see gnu.gpl.Licence
+ * @see gnu.gpl.License for license details.
+ *      The copyright to this program is held by it's authors.
  * @author Joe Walker [joe at eireneh dot com]
  * @author Mark Goodwin [mark at thorubio dot org]
  * @author DM Smith [dmsmith555 at yahoo dot com]
- * @version $Id: Desktop.java,v 1.38 2005/03/21 02:37:18 dmsmith Exp $
  */
-public class Desktop extends JFrame implements URLEventListener, ViewEventListener, ViewGenerator
+public class Desktop extends JFrame implements URLEventListener, ViewEventListener, DisplaySelectListener, ViewGenerator
 {
     /**
      * Central start point.
@@ -118,17 +124,26 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
                     ExceptionPane.setHelpDeskListener(true);
                     LookAndFeelUtil.initialize();
 
-                    Desktop desktop = new Desktop();
+                    final Splash splash = new Splash();
+
+                    final Desktop desktop = new Desktop();
 
                     // change the size and location before showing the application.
                     GuiUtil.setSize(desktop, getDefaultSize());
                     GuiUtil.centerWindow(desktop);
 
-                    // Don't use pack.
-                    // It uses preferred dimensions, which are not used here.
-                    desktop.show();
-                    desktop.toFront();
-                    desktop.setVisible(true);
+                    // Now bring up the app and offer to install books if the user has none.
+                    SwingUtilities.invokeLater(new Runnable()
+                        {
+                            public void run()
+                            {
+                                splash.close();
+                                desktop.setVisible(true);
+                                desktop.establishPreferredSize();
+                                desktop.pack();
+                                desktop.checkForBooks();
+                            }
+                        });
                 }
             };
             t.start();
@@ -153,11 +168,35 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
         // ResourceBundles, properties and other resources
         Project project = Project.instance();
 
+        // Splash screen
+        URL predicturl = project.getWritablePropertiesURL(SPLASH_PROPS);
+        Job startJob = JobManager.createJob(Msg.STARTUP_TITLE.toString(), predicturl, true);
+
         // Load the configuration.
-        // This has to be done before any gui components are created are created.
+        // This has to be done before any gui components are created.
+        // (Other than the splash)
         // This includes code that is invoked by it.
+        startJob.setProgress(Msg.STARTUP_CONFIG.toString());
         generateConfig();
 
+        // Make this be the root frame of optiondialogs
+        JOptionPane.setRootFrame(this);
+
+        // Grab errors
+        Reporter.grabAWTExecptions(true);
+
+        // Create the Desktop Actions
+        actions = new DesktopActions(this);
+
+        startJob.setProgress(Msg.STARTUP_GENERATE.toString());
+        createComponents();
+
+        // Configuration
+        startJob.setProgress(Msg.STARTUP_GENERAL_CONFIG.toString());
+        // GUI setup
+        debug();
+        init();
+
         // Listen for book changes so that the Options can be kept current
         BooksListener cbl = new BooksListener()
         {
@@ -173,52 +212,7 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
         };
         Books.installed().addBooksListener(cbl);
 
-        // Make this be the root frame of optiondialogs
-        JOptionPane.setRootFrame(this);
-
-        // Grab errors
-        Reporter.grabAWTExecptions(true);
-
-        // Splash screen
-        URL predicturl = project.getWritablePropertiesURL(SPLASH_PROPS);
-        Splash splash = new Splash();
-        Job startJob = JobManager.createJob(Msg.STARTUP_TITLE.toString(), predicturl, true);
-        splash.pack();
-
-        // Create the Desktop Actions
-        actions = new DesktopActions(this);
-
-        startJob.setProgress(Msg.STARTUP_CONFIG.toString());
-
-        startJob.setProgress(Msg.STARTUP_GENERATE.toString());
-        createComponents();
-
-        // GUI setup
-        debug();
-        init();
-
-        // This is technically overkill, but it does hide the reference pane if
-        // there are no reference works
-        refreshBooks();
-
-        // Configuration
-        startJob.setProgress(Msg.STARTUP_GENERAL_CONFIG.toString());
-
         startJob.done();
-        splash.close();
-
-        // News users probably wont have any Bibles installedso we give them a
-        // hand getting to the installation diallog.
-        List bibles = Books.installed().getBooks(BookFilters.getBibles());
-        if (bibles.size() == 0)
-        {
-            int reply = JOptionPane.showConfirmDialog(this, Msg.NO_BIBLES_MESSAGE, Msg.NO_BIBLES_TITLE.toString(), JOptionPane.OK_CANCEL_OPTION,
-                            JOptionPane.QUESTION_MESSAGE);
-            if (reply == JOptionPane.OK_OPTION)
-            {
-                actions.doBooks();
-            }
-        }
     }
 
     /**
@@ -238,19 +232,125 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
     private void createComponents()
     {
         barStatus = new StatusBar();
-        pnlTbar = new ToolBar(this);
         //barSide = new SidebarPane();
         //barBook = new ReferencedPane();
         reference = new DictionaryPane();
-        sptBooks = new FixedSplitPane();
+        sptBooks = new FixedSplitPane(false);
         views = new ViewManager(this);
         views.addViewEventListener(this);
+        history = new History();
     }
 
     /**
      * Initialize the GUI, and display it.
      */
     private void init()
+    {
+        addWindowListener(new WindowAdapter()
+        {
+            public void windowClosed(WindowEvent ev)
+            {
+                actions.getAction(DesktopActions.EXIT).actionPerformed(new ActionEvent(this, 0, EMPTY_STRING));
+            }
+        });
+
+        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
+
+        TDIViewLayout tdi = (TDIViewLayout) LayoutType.TDI.getLayout();
+        tdi.addPopup(createPopupMenu());
+
+        //barBook.addHyperlinkListener(this);
+        //barSide.addHyperlinkListener(this);
+        reference.addURLEventListener(this);
+
+        sptBooks.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
+        sptBooks.setRightComponent(reference);
+        sptBooks.setLeftComponent(views.getDesktop());
+        sptBooks.setResizeWeight(0.8D);
+        sptBooks.setOpaque(true);
+        sptBooks.setBorder(null);
+
+        // The toolbar needs to be in the outermost container, on the border
+        // And the only other item in that container can be CENTER
+        JComponent contentPane = (JComponent) getContentPane();
+        contentPane.setLayout(new BorderLayout());
+        ToolBar toolbar = createToolBar();
+        contentPane.add(toolbar, BorderLayout.NORTH);
+
+        JPanel mainPanel = new JPanel(new BorderLayout());
+        mainPanel.setBorder(BorderFactory.createEmptyBorder(1, 5, 1, 5));
+        mainPanel.add(sptBooks, BorderLayout.CENTER);
+
+        // Put everything else in its own panel
+        corePanel = new JPanel(new BorderLayout());
+        corePanel.add(mainPanel, BorderLayout.CENTER);
+        corePanel.add(barStatus, BorderLayout.SOUTH);
+        contentPane.add(corePanel, BorderLayout.CENTER);
+        setJMenuBar(createMenuBar(toolbar));
+
+        setIconImage(ICON_APP.getImage());
+        setEnabled(true);
+        setTitle(Msg.getApplicationTitle());
+    }
+
+    private JMenuBar createMenuBar(ToolBar toolbar)
+    {
+        JMenuBar barMenu = new JMenuBar();
+        barMenu.add(createFileMenu());
+        barMenu.add(createEditMenu());
+        barMenu.add(createViewMenu(toolbar));
+        barMenu.add(createGoMenu());
+        barMenu.add(createToolsMenu());
+        barMenu.add(createHelpMenu());
+        return barMenu;
+    }
+
+    private JPopupMenu createPopupMenu()
+    {
+        JPopupMenu popup = new JPopupMenu();
+        popup.add(views.getContextAction(ViewManager.NEW_TAB)).addMouseListener(barStatus);
+        popup.add(views.getContextAction(ViewManager.CLOSE_VIEW)).addMouseListener(barStatus);
+        popup.add(views.getContextAction(ViewManager.CLEAR_VIEW)).addMouseListener(barStatus);
+        popup.add(views.getContextAction(ViewManager.CLOSE_OTHER_VIEWS)).addMouseListener(barStatus);
+        popup.add(views.getContextAction(ViewManager.CLOSE_ALL_VIEWS)).addMouseListener(barStatus);
+        popup.addSeparator();
+        popup.add(actions.getAction(DesktopActions.OPEN)).addMouseListener(barStatus);
+        popup.add(actions.getAction(DesktopActions.SAVE)).addMouseListener(barStatus);
+        popup.add(actions.getAction(DesktopActions.SAVE_AS)).addMouseListener(barStatus);
+        popup.add(actions.getAction(DesktopActions.SAVE_ALL)).addMouseListener(barStatus);
+        return popup;
+    }
+
+    private ToolBar createToolBar()
+    {
+        ToolBar toolbar = new ToolBar(this);
+        toolbar.setRollover(true);
+        toolbar.setFloatable(true);
+
+        toolbar.add(views.getContextAction(ViewManager.NEW_TAB)).addMouseListener(barStatus);
+        toolbar.add(actions.getAction(DesktopActions.OPEN)).addMouseListener(barStatus);
+        toolbar.add(actions.getAction(DesktopActions.SAVE)).addMouseListener(barStatus);
+        toolbar.addSeparator();
+        //toolbar.add(actions.getAction(DesktopActions.CUT)).addMouseListener(barStatus);
+        toolbar.add(actions.getAction(DesktopActions.COPY)).addMouseListener(barStatus);
+        //toolbar.add(actions.getAction(DesktopActions.PASTE)).addMouseListener(barStatus);
+        toolbar.addSeparator();
+        toolbar.add(actions.getAction(DesktopActions.BACK)).addMouseListener(barStatus);
+        toolbar.add(actions.getAction(DesktopActions.FORWARD)).addMouseListener(barStatus);
+        toolbar.addSeparator();
+        //toolbar.add(actions.getAction("Generate")).addMouseListener(barStatus);
+        //toolbar.add(actions.getAction("Diff")).addMouseListener(barStatus);
+        //toolbar.addSeparator();
+        toolbar.add(actions.getAction(DesktopActions.CONTENTS)).addMouseListener(barStatus);
+        toolbar.add(actions.getAction(DesktopActions.ABOUT)).addMouseListener(barStatus);
+
+        return toolbar;
+    }
+    /**
+     * Create the file menu
+     * @return the file menu
+     */
+    private JMenu createFileMenu()
     {
         JMenu menuFile = new JMenu(actions.getAction(DesktopActions.FILE));
         menuFile.add(views.getContextAction(ViewManager.NEW_TAB)).addMouseListener(barStatus);
@@ -269,68 +369,98 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
         menuFile.addSeparator();
         menuFile.add(actions.getAction(DesktopActions.EXIT)).addMouseListener(barStatus);
         menuFile.setToolTipText(null);
+        return menuFile;
+    }
 
-        JPopupMenu popup = new JPopupMenu();
-        popup.add(views.getContextAction(ViewManager.NEW_TAB)).addMouseListener(barStatus);
-        popup.add(views.getContextAction(ViewManager.CLOSE_VIEW)).addMouseListener(barStatus);
-        popup.add(views.getContextAction(ViewManager.CLEAR_VIEW)).addMouseListener(barStatus);
-        popup.add(views.getContextAction(ViewManager.CLOSE_OTHER_VIEWS)).addMouseListener(barStatus);
-        popup.add(views.getContextAction(ViewManager.CLOSE_ALL_VIEWS)).addMouseListener(barStatus);
-        popup.addSeparator();
-        popup.add(actions.getAction(DesktopActions.OPEN)).addMouseListener(barStatus);
-        popup.add(actions.getAction(DesktopActions.SAVE)).addMouseListener(barStatus);
-        popup.add(actions.getAction(DesktopActions.SAVE_AS)).addMouseListener(barStatus);
-        popup.add(actions.getAction(DesktopActions.SAVE_ALL)).addMouseListener(barStatus);
-
-        TDIViewLayout tdi = (TDIViewLayout) LayoutType.TDI.getLayout();
-        tdi.addPopup(popup);
-
+    private JMenu createEditMenu()
+    {
         JMenu menuEdit = new JMenu(actions.getAction(DesktopActions.EDIT));
         //menuEdit.add(actions.getAction(DesktopActions.CUT)).addMouseListener(barStatus);
         menuEdit.add(actions.getAction(DesktopActions.COPY)).addMouseListener(barStatus);
         //menuEdit.add(actions.getAction(DesktopActions.PASTE)).addMouseListener(barStatus);
         menuEdit.setToolTipText(null);
+        return menuEdit;
+    }
 
+    private JMenu createGoMenu()
+    {
+        JMenu menuGo = new JMenu(actions.getAction(DesktopActions.GO));
+        menuGo.add(actions.getAction(DesktopActions.BACK)).addMouseListener(barStatus);
+        menuGo.add(actions.getAction(DesktopActions.FORWARD)).addMouseListener(barStatus);
+        return menuGo;
+    }
+    /**
+     * Create the view menu.
+     * @return the view menu.
+     */
+    private JMenu createViewMenu(ToolBar toolbar)
+    {
         JMenu menuView = new JMenu(actions.getAction(DesktopActions.VIEW));
-        JCheckBoxMenuItem toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.STRONGS_NUMBERS.getName()));
-        toggle.setSelected(XSLTProperty.STRONGS_NUMBERS.getDefault());
+        JCheckBoxMenuItem toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.TINY_VERSE_NUMBERS.getName()));
+        toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.TINY_VERSE_NUMBERS.getName()));
+        toggle.setSelected(XSLTProperty.TINY_VERSE_NUMBERS.getDefault());
         menuView.add(toggle).addMouseListener(barStatus);
         toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.START_VERSE_ON_NEWLINE.getName()));
         toggle.setSelected(XSLTProperty.START_VERSE_ON_NEWLINE.getDefault());
         menuView.add(toggle).addMouseListener(barStatus);
-        toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.VERSE_NUMBERS.getName()));
-        toggle.setSelected(XSLTProperty.VERSE_NUMBERS.getDefault());
-        menuView.add(toggle).addMouseListener(barStatus);
-        toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.TINY_VERSE_NUMBERS.getName()));
-        toggle.setSelected(XSLTProperty.TINY_VERSE_NUMBERS.getDefault());
-        menuView.add(toggle).addMouseListener(barStatus);
+        JMenu verseMenu = new JMenu(actions.getAction(DesktopActions.VERSE));
+        menuView.add(verseMenu);
+        ButtonGroup grpNumbering = new ButtonGroup();
+        JRadioButtonMenuItem radio = new JRadioButtonMenuItem(actions.getAction(XSLTProperty.VERSE_NUMBERS.getName()));
+        grpNumbering.add(radio);
+        radio.setSelected(XSLTProperty.VERSE_NUMBERS.getDefault());
+        verseMenu.add(radio).addMouseListener(barStatus);
+        radio = new JRadioButtonMenuItem(actions.getAction(XSLTProperty.CV.getName()));
+        grpNumbering.add(radio);
+        radio.setSelected(XSLTProperty.CV.getDefault());
+        verseMenu.add(radio).addMouseListener(barStatus);
+        radio = new JRadioButtonMenuItem(actions.getAction(XSLTProperty.BCV.getName()));
+        grpNumbering.add(radio);
+        radio.setSelected(XSLTProperty.BCV.getDefault());
+        verseMenu.add(radio).addMouseListener(barStatus);
+        radio = new JRadioButtonMenuItem(actions.getAction(XSLTProperty.NO_VERSE_NUMBERS.getName()));
+        grpNumbering.add(radio);
+        radio.setSelected(XSLTProperty.NO_VERSE_NUMBERS.getDefault());
+        verseMenu.add(radio).addMouseListener(barStatus);
+
+        menuView.addSeparator();
         toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.NOTES.getName()));
         toggle.setSelected(XSLTProperty.NOTES.getDefault());
         menuView.add(toggle).addMouseListener(barStatus);
         toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.XREF.getName()));
         toggle.setSelected(XSLTProperty.XREF.getDefault());
         menuView.add(toggle).addMouseListener(barStatus);
+        toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.STRONGS_NUMBERS.getName()));
+        toggle.setSelected(XSLTProperty.STRONGS_NUMBERS.getDefault());
+        menuView.add(toggle).addMouseListener(barStatus);
+        toggle = new JCheckBoxMenuItem(actions.getAction(XSLTProperty.MORPH.getName()));
+        toggle.setSelected(XSLTProperty.MORPH.getDefault());
+        menuView.add(toggle).addMouseListener(barStatus);
         menuView.addSeparator();
         menuView.add(views.getTdiView()).addMouseListener(barStatus);
         menuView.add(views.getMdiView()).addMouseListener(barStatus);
         //menuView.add(chkViewTbar);
         menuView.addSeparator();
-        menuView.add(pnlTbar.getShowToggle()).addMouseListener(barStatus);
-        menuView.add(pnlTbar.getTextToggle()).addMouseListener(barStatus);
-        menuView.add(pnlTbar.getIconSizeToggle()).addMouseListener(barStatus);
+        menuView.add(toolbar.getShowToggle()).addMouseListener(barStatus);
+        menuView.add(toolbar.getTextToggle()).addMouseListener(barStatus);
+        menuView.add(toolbar.getIconSizeToggle()).addMouseListener(barStatus);
         toggle = new JCheckBoxMenuItem(actions.getAction(DesktopActions.TOOLTIP_TOGGLE));
         toggle.setSelected(true);
         menuView.add(toggle).addMouseListener(barStatus);
         toggle = new JCheckBoxMenuItem(actions.getAction(DesktopActions.STATUS_TOGGLE));
         toggle.setSelected(true);
         menuView.add(toggle).addMouseListener(barStatus);
-        toggle = new JCheckBoxMenuItem(actions.getAction(DesktopActions.SIDEBAR_TOGGLE));
-        toggle.setSelected(true);
-        menuView.add(toggle).addMouseListener(barStatus);
+        sidebarToggle = new JCheckBoxMenuItem(actions.getAction(DesktopActions.SIDEBAR_TOGGLE));
+        sidebarToggle.setSelected(isSidebarShowing());
+        menuView.add(sidebarToggle).addMouseListener(barStatus);
         menuView.addSeparator();
         menuView.add(actions.getAction(DesktopActions.VIEW_SOURCE)).addMouseListener(barStatus);
         menuView.setToolTipText(null);
+        return menuView;
+    }
 
+    private JMenu createToolsMenu()
+    {
         JMenu menuTools = new JMenu(actions.getAction(DesktopActions.TOOLS));
         //menuTools.add(actions.getAction(DesktopActions.GENERATE)).addMouseListener(barStatus);
         //menuTools.add(actions.getAction(DesktopActions.DIFF)).addMouseListener(barStatus);
@@ -338,76 +468,27 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
         menuTools.add(actions.getAction(DesktopActions.BOOKS)).addMouseListener(barStatus);
         menuTools.add(actions.getAction(DesktopActions.OPTIONS)).addMouseListener(barStatus);
         menuTools.setToolTipText(null);
+        return menuTools;
+    }
 
+    private JMenu createHelpMenu()
+    {
         JMenu menuHelp = new JMenu(actions.getAction(DesktopActions.HELP));
         menuHelp.add(actions.getAction(DesktopActions.CONTENTS)).addMouseListener(barStatus);
         menuHelp.addSeparator();
         menuHelp.add(actions.getAction(DesktopActions.ABOUT)).addMouseListener(barStatus);
         menuHelp.setToolTipText(null);
+        return menuHelp;
+    }
+    /**
+     * Get the size of the content panel and make that the preferred size.
+     */
+    public void establishPreferredSize()
+    {
+        JComponent contentPane = (JComponent) getContentPane();
+        contentPane.setPreferredSize(contentPane.getSize());
 
-        JMenuBar barMenu = new JMenuBar();
-        barMenu.add(menuFile);
-        barMenu.add(menuEdit);
-        barMenu.add(menuView);
-        barMenu.add(menuTools);
-        barMenu.add(menuTools);
-        barMenu.add(menuHelp);
-
-        pnlTbar.setRollover(true);
-        pnlTbar.setFloatable(true);
-
-        pnlTbar.add(views.getContextAction(ViewManager.NEW_TAB)).addMouseListener(barStatus);
-        pnlTbar.add(actions.getAction(DesktopActions.OPEN)).addMouseListener(barStatus);
-        pnlTbar.add(actions.getAction(DesktopActions.SAVE)).addMouseListener(barStatus);
-        pnlTbar.addSeparator();
-        //pnlTbar.add(actions.getAction(DesktopActions.CUT)).addMouseListener(barStatus);
-        pnlTbar.add(actions.getAction(DesktopActions.COPY)).addMouseListener(barStatus);
-        //pnlTbar.add(actions.getAction(DesktopActions.PASTE)).addMouseListener(barStatus);
-        pnlTbar.addSeparator();
-        //pnlTbar.add(actions.getAction("Generate")).addMouseListener(barStatus);
-        //pnlTbar.add(actions.getAction("Diff")).addMouseListener(barStatus);
-        //pnlTbar.addSeparator();
-        pnlTbar.add(actions.getAction(DesktopActions.CONTENTS)).addMouseListener(barStatus);
-        pnlTbar.add(actions.getAction(DesktopActions.ABOUT)).addMouseListener(barStatus);
-
-        //barBook.addHyperlinkListener(this);
-        //barSide.addHyperlinkListener(this);
-        reference.addURLEventListener(this);
-
-        sptBooks.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
-        sptBooks.setRightComponent(reference);
-        sptBooks.setLeftComponent(views.getDesktop());
-        sptBooks.setResizeWeight(0.8D);
-        sptBooks.setDividerSize(7);
-        sptBooks.setOpaque(true);
-        sptBooks.setBorder(null);
-
-        addWindowListener(new WindowAdapter()
-        {
-            public void windowClosed(WindowEvent ev)
-            {
-                actions.getAction(DesktopActions.EXIT).actionPerformed(new ActionEvent(this, 0, EMPTY_STRING));
-            }
-        });
-
-        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
-
-        // The toolbar needs to be in the outermost container, on the border
-        // And the only other item in that container can be CENTER
-        final JComponent contentPane = (JComponent) getContentPane();
-        contentPane.setLayout(new BorderLayout());
-        contentPane.add(pnlTbar, BorderLayout.NORTH);
-
-        // Put everything else in its own panel
-        corePanel = new JPanel(new BorderLayout());
-        corePanel.add(barStatus, BorderLayout.SOUTH);
-        corePanel.add(sptBooks, BorderLayout.CENTER);
-        contentPane.add(corePanel, BorderLayout.CENTER);
-        setJMenuBar(barMenu);
-
-        setIconImage(ICON_APP.getImage());
-        setEnabled(true);
-        setTitle(Msg.getApplicationTitle());
+        log.warn("The size of the contentpane is: " + contentPane.getSize()); //$NON-NLS-1$
     }
 
     /**
@@ -420,10 +501,13 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
 
     public Component createView()
     {
-        BibleViewPane view = new BibleViewPane();
+        boolean show = sidebarToggle == null ? isSidebarShowing() : sidebarToggle.isSelected();
+        BibleViewPane view = new BibleViewPane(show);
         BookDataDisplay display = view.getPassagePane().getBookDataDisplay();
         display.addURLEventListener(this);
         display.addURLEventListener(barStatus);
+        DisplaySelectPane dsp = view.getSelectPane();
+        dsp.addCommandListener(this);
         return view;
     }
 
@@ -436,6 +520,38 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
         BookDataDisplay display = view.getPassagePane().getBookDataDisplay();
         display.removeURLEventListener(this);
         display.removeURLEventListener(barStatus);
+        DisplaySelectPane dsp = view.getSelectPane();
+        dsp.removeCommandListener(this);
+    }
+
+    /* (non-Javadoc)
+     * @see org.crosswire.bibledesktop.book.DisplaySelectListener#bookChosen(org.crosswire.bibledesktop.book.DisplaySelectEvent)
+     */
+    public void bookChosen(DisplaySelectEvent ev)
+    {
+        // Do nothing
+    }
+
+    /* (non-Javadoc)
+     * @see org.crosswire.bibledesktop.book.DisplaySelectListener#passageSelected(org.crosswire.bibledesktop.book.DisplaySelectEvent)
+     */
+    public void passageSelected(DisplaySelectEvent ev)
+    {
+        Key key = ev.getKey();
+        if (key != null && !key.isEmpty())
+        {
+            // add the string because keys are heavyweights
+            history.add(key.getName());
+        }
+    }
+
+    public void selectHistory(int i)
+    {
+        Object obj = history.go(i);
+        if (obj != null)
+        {
+            activateURL(new URLEvent(this, Desktop.BIBLE_PROTOCOL, (String) obj));
+        }
     }
 
     /* (non-Javadoc)
@@ -445,7 +561,7 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
     {
         barStatus.activateURL(ev);
         String protocol = ev.getProtocol();
-        String data = ev.getUrl();
+        String data = ev.getURL();
 
         try
         {
@@ -476,6 +592,17 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
                     }
                 }
 
+                // If we didn't find a view and BibleViews are reused,
+                // then pretend that the selected view is clear.
+                if (isBibleViewReused())
+                {
+                    BibleViewPane view = (BibleViewPane) views.getSelected();
+                    if (view != null)
+                    {
+                        clearView = view;
+                    }
+                }
+
                 // Do we have an empty view we can use?
                 if (clearView != null)
                 {
@@ -483,6 +610,7 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
                     if (book != null)
                     {
                         Key key = book.getKey(data);
+                        clearView.setKey(book.createEmptyKeyList()); // force it to be a clear view, if it is not really.
                         clearView.setKey(key);
                         views.select(clearView);
                     }
@@ -647,6 +775,54 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
 
     }
 
+    public void checkForBooks()
+    {
+        // News users probably wont have any Bibles installed so we give them a
+        // hand getting to the installation dialog.
+        List bibles = Books.installed().getBooks(BookFilters.getBibles());
+        if (bibles.size() == 0)
+        {
+            int reply = JOptionPane.showConfirmDialog(this, Msg.NO_BIBLES_MESSAGE, Msg.NO_BIBLES_TITLE.toString(), JOptionPane.OK_CANCEL_OPTION,
+                            JOptionPane.QUESTION_MESSAGE);
+            if (reply == JOptionPane.OK_OPTION)
+            {
+                actions.doBooks();
+            }
+        }
+    }
+
+    /**
+     * @param show Whether to show the KeySidebar at start up.
+     */
+    public static void setSidebarShowing(boolean show)
+    {
+        sidebarShowing = show;
+    }
+
+    /**
+     * @return Whether to show the KeySidebar at start up.
+     */
+    public static boolean isSidebarShowing()
+    {
+        return sidebarShowing;
+    }
+
+    /**
+     * @param reuse Whether reuse the current BibleView.
+     */
+    public static void setBibleViewReused(boolean reuse)
+    {
+        reuseBibleView = reuse;
+    }
+
+    /**
+     * @return Whether links use the current BibleView.
+     */
+    public static boolean isBibleViewReused()
+    {
+        return reuseBibleView;
+    }
+
     /**
      * @param maxHeight The maxHeight to set.
      */
@@ -781,7 +957,17 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
     /**
      * The configuration engine
      */
-    private Config config;
+    private transient Config config;
+
+    /**
+     * Whether to show the Key Sidebar at startup
+     */
+    private static boolean sidebarShowing;
+
+    /**
+     * Whether to current BibleView should be used for links
+     */
+    private static boolean reuseBibleView = true;
 
     /**
      * The default dimension for this frame
@@ -793,19 +979,20 @@ public class Desktop extends JFrame implements URLEventListener, ViewEventListen
      */
     protected static final Logger log = Logger.getLogger(Desktop.class);
 
-    protected DesktopActions actions;
+    protected transient DesktopActions actions;
 
     /**
      * The application icon
      */
     private static final ImageIcon ICON_APP = GuiUtil.getIcon("images/icon16.png"); //$NON-NLS-1$
 
-    private ViewManager views;
+    private transient ViewManager views;
     private JPanel corePanel;
-    private ToolBar pnlTbar;
+    private JCheckBoxMenuItem sidebarToggle;
     private StatusBar barStatus;
     private DictionaryPane reference;
     private JSplitPane sptBooks;
+    private transient History history;
 
     /**
      * Serialization ID
